% Shifth through version ST.m
% author: Matthias Humet

% revised: 25-05-2011 (check if U(2,1) and U(3,1) simultaneously zero)

function [ H1,H2,H3] = ST( G1,G2,G3,dotest)
%ST Given 3 givens rotations G1,G2,G3, this function computes 3 givens
%rotations H1,H2,H3 such that:
%       blkdiag(1,H1)*blkdiag(H2,1)*blkdiag(1,H3) 
%       ==
%       blkdiag(G1,1)*blkdiag(1,G2)*blkdiag(G3,1)
%
% If G1,G2,G3 satisfy (*)
%   * G1 is an arbitrary 2 x 2 unitary matrix
%   * G2, G3 are of type [c s; s -c'] with s positive real
% Then H1,H2,H3 satisfy
%   * H1, H2 are of type [c s; s -c'] with s positive real
%   * H3 is an arbitrary 2 x 2 unitary matrix
% 
% PROOF:
%   Let U = G1.G2.G3. U(3,1) will be positive real under (*). In that case
%   givens_alg gives H1 with s positive real. Now U(2,1) is positive real.
%   With this, givens_alg gives H2 with s positive real. What remains in
%   U(2:3,2:3) = H3 is an arbitrary unitary matrix.
%  

test = false;
if nargin > 3
    test = dotest;
end

% U = eye(3);
% U(1:2,:) = G3*U(1:2,:);
U = [G3 [0;0]; 0 0 1]; % in order for vpa to work
U(2:3,:) = G2*U(2:3,:);
U(1:2,:) = G1*U(1:2,:); 

if test
	Uorig = U;
end

if U(2,1) == 0 && U(3,1) == 0
    H1 = eye(2);
else
    H1 = givens_alg(U(2,1),U(3,1));
    U(2:3,:) = H1*U(2:3,:);
    H1 = H1';
end
    
H2 = givens_alg(U(1,1),U(2,1));
U(1:2,:) = H2*U(1:2,:);
% H2(1,:) = sign(U(1,1))'*H2(1,:); not necessary, since givens_alg gives positive
% real r.
H2 = H2';

H3 = U(2:3,2:3);

if test
	B=eye(3);
	B(2:3,:)=H3*B(2:3,:);
	B(1:2,:)=H2*B(1:2,:);
	B(2:3,:)=H1*B(2:3,:);
	Uorig
	norm(B-Uorig)
	abs((Uorig-B)./Uorig)
	% pause
end

end

