% author: Matthias Humet
% 
% version of paper: may 20th 2011.
% 24-11-11:
%   added: check if value of c is valid

function [ Ghat ] = geronimus_fwd_direct( G, alpha, a1 )
%GEROMIMUS_FWD This is an exact implementation of Algorithm 8.1 (fwd,
%direct approach) of the paper.

n = length(G);
Ghat = cell(n+1,1);
rho = sqrt(1-abs(a1)^2);
Ghat{1} = [-a1 rho; rho a1'];

r11 = sqrt(1+abs(alpha)^2+2*real(alpha*a1'));
c = - (a1'+alpha')/r11;
s = sqrt(1-abs(c)^2);
Q1 = [c' s; s -c];
K = G{1};
v = [r11; 0];
q = [alpha; 0];
j = 2;

a = a1;

message_c = 1;

for k = 1:n
    r = (K(:,1) - q - c'*v)/s;
    v = s*v-c*r;
    c = (v(1)*K(j,2)/K(1,2)-v(j))/alpha;
    if message_c && double(abs(c)) >= 1
        fprintf('\n*** In geronimus_fwd_direct:\n  step %i, |c| >= 1\n',k)
        message_c = 0;
    end
    s = sqrt(1-abs(c)^2);
    Q2 = [c' s; s -c];
    y = givprod([r(j-1);r(j);0],Q2,2,Q1,1) + [0; alpha; 0];
    a = -y(2)/a'; rho = y(3);
    Ghat{k+1} = [-a rho; rho a'];
    if k < n
        K = [ K(1,2) 0; K(j,2) 0; 0 1 ]*G{k+1};
        v = [v(1); v(j); 0];
        q = [0; alpha; 0];
        j = 3;
        
        Q1 = Q2;
    end
end

