% author: Matthias Humet
% 
% version of paper: may 20th 2011.
% 19-11-11: 
%   added: conditional line to make c1 computed in DP if inputs are all in
%   VPA.
%
% 24-11-11:
%   added: check if value of c is valid
% 12-12-11:
%   added: give back k at which value of c is no longer valid

function [ Ghat, k_breakdown ] = geronimus_fwd_st( G, alpha, a1, c1inDP )
%GERONIMUS_FWD_ST This is an exact implementation of Algorithm 8.2 (fwd,
% shift through) of the paper.
%
% 

n = length(G);
Ghat = cell(n+1,1);
rho = sqrt(1-abs(a1)^2);
Ghat{1} = [-a1 rho; rho a1'];

r11 = sqrt(1+abs(alpha)^2+2*real(alpha*a1'));
% if a1 and alpha in VPA and c1inDP = 1, compute c1 in DP
if isa(alpha,'sym') && isa(a1,'sym') && nargin > 3 && c1inDP
	c = vpa(- (double(a1)'+double(alpha)')/double(r11));
else
    c = - (a1'+alpha')/r11;
end
s = sqrt(1-abs(c)^2);
Q = [c' s; s -c];

K = G{1};
v = [r11; 0];
q = [alpha; 0];

E = Ghat{1}'*Q;

message_c = 1;
k_breakdown = -1;

for k = 1:n
    r = (K(:,1) - q - c'*v)/s;
    v = s*v-c*r;
    c = (v(1)*K(2,2)/K(1,2)-v(2))/alpha;
    if message_c && double(abs(c)) >= 1
        fprintf('\n*** In geronimus_fwd_st:\n  step %i, |c| >= 1\n',k)
        k_breakdown = k;
        message_c = 0;
    end
    s = sqrt(1-abs(c)^2);
    Q = [c' s; s -c];
    [Ghat{k+1},X,E] = ST(E,Q,G{k});
    if k < n
        K = [K(1,2) 0; 0 1]*G{k+1};
        v = [v(1); 0];
        q = [0; 0];
    end    
end

