% author: Matthias Humet
%
%
function [ B ] = givprod( A, G, k, varargin )
%GIVPROD Apply 2x2 Givens G to the k-th and (k+1)-th rows of A, yielding B.
%
% B = givprod(A,G1,k1,G2,k2,G3,k3,...) applies first G1 at position k1,
% then G2 at position k2, etc.


assert(sum(size(G) == [2 2]) == 2,' ** error in givprod.m: G must be Givens.')
assert(size(A,1) > k,' ** error in givprod.m: A must have at least k+1 rows.')

B = A;
B(k:k+1,:) = G*B(k:k+1,:);

if nargin - 3 >= 2
    G = varargin{1};
    k = varargin{2};
    B = givprod(B,G,k,varargin{3:end});
end


end

