% 12-12-11
% author: Matthias Humet
%
% This script containts the code to make the plots of the paper "Algorithms
% for the Geronimus transformations for orthogonal polynomials on the unit
% circle."

fprintf('\n\n START OF MAIN_PAPER_ALL.m:')
fprintf('\n==========================\n')

digits(256)

%% 1. Backward accuracy
% Here, we generate plots to show the accuracy of the backward
% Geronimus algorithm, or equivalently, of the Christoffel algorithm.

fprintf('\n\n 1. Backward accuracy:')
fprintf('\n==========================\n\n')

n = 200;

alpha = 1.2*exp(i*pi/5);
a = rand(n,1).*exp(i*2*pi*rand(n,1));
G = givensfactors(a);
Ghat = geronimus_bwd(givensfactors(a),alpha);
fprintf('\n VPA geronimus_bwd ...')
tic
Ghatv = geronimus_bwd(givensfactors(vpa(a)),vpa(alpha));
fprintf('  %0.3f seconds.\n',toc)
err = abs(reflection(Ghat)-reflection(Ghatv));

figure(11),clf
semilogy(err,'*')
xlabel('k','fontsize',14)
ylabel('|a_k-b_k|','fontsize',14)
set(gca,'fontsize',14)
% set(get(11,'Children'),'fontname','arial')%,'fontsize',16)
saveas(11,'geronimus_bwd_acc','epsc')

%% 2. Backward and forward 
% Here, we make some plots of errors and convergence of the backward
% algorithm. We additionally plot some errors for the forward algorithms on
% the same sequences. 
    
fprintf('\n\n 2. Backward and forward :')
fprintf('\n==========================\n\n')

n=100;

% Finish with choice = 1, because we use the same inputs in the next cell
for choice = 2:-1:1 

switch choice 
    case 1
        alpha = [1.001 1.3 2.1]*exp(i*pi/5);
        a = 0.1*ones(n-1,1);
        choice_string = '_a=0p1';
    case 2
        alpha = [1.001 1.3 2.1]*exp(i*pi/5);
        a = 0.6*ones(n-1,1); 
        choice_string = '_a=0p6';
    case 3
        alpha = [1.001 1.3 2.1]*exp(i*pi/5);
        a = rand(n-1,1).*exp(i*2*pi*rand(n-1,1));
        choice_string = '_a=rand_complex';        
end

an = [-0.95, 0.5+0.5*i];
G = givensfactors(a);
G1 = givensfactors([a;an(1)]);
G2 = givensfactors([a;an(2)]);

diff_Gbwd = zeros(n,length(alpha));
err_Gfwd = zeros(n,length(alpha));
err_Gfwd_direct = zeros(n,length(alpha));

for k = 1:length(alpha)
    Gbwd1 = geronimus_bwd(G1,alpha(k));
    Gbwd2 = geronimus_bwd(G2,alpha(k));
    diff_Gbwd(:,k) = abs(reflection(Gbwd1)-reflection(Gbwd2));
    
    Gfwd = geronimus_fwd_st(G,alpha(k),-Gbwd1{1}(1,1));
    Gfwd_direct = geronimus_fwd_direct(G,alpha(k),-Gbwd1{1}(1,1));
    err_Gfwd(:,k) = abs(reflection(Gbwd1)-reflection(Gfwd));
    err_Gfwd_direct(:,k) = abs(reflection(Gbwd1)-reflection(Gfwd_direct));
end

% -----------
% --- plots: 
% -----------

alpha_strings = cell(length(alpha),1);
for k = 1:length(alpha)
    alpha_strings{k} = ['|\alpha| = ',sprintf('%0.3f',abs(alpha(k)))];
end
alpha_strings_bwd = alpha_strings;

alpha_strings_fwd = {alpha_strings{:}, alpha_strings{:}};
for k = 1:length(alpha)
    alpha_strings_fwd{k} = [alpha_strings_fwd{k},' (ST)'];
    alpha_strings_fwd{k+length(alpha)} = [alpha_strings_fwd{k+length(alpha)},' (direct)'];
end

figure(21),clf
h1 = semilogy(diff_Gbwd,'linewidth',2);
setlinestyles(h1,{'-','--','-.'})
hold on
set(gca,'fontsize',16)
legend(alpha_strings_bwd,'location','best')
xlabel('k','fontsize',16)
ylabel('|a_k-b_k|','fontsize',16)
axis([0 100 1e-18 1e4])
saveas(21,['geronimus_bwd_3alpha',choice_string],'epsc')

% Plotting comments:
% * semilogy plots with different colors, but we need to set the linestyles
% manually. (Matlab doesn't cycle through colors and linestyles together.)
% * h4 are the same 3 lines as h3, but only the markers, at a distance
% step from each other.
% * To make the legend correct, we plot h_aux after h_2 with style '-o',
% and set its linestyles. We want in the legend to appear
% '..o..','--o--',etc.
figure(22),clf
h2 = semilogy(err_Gfwd,'linewidth',2);
setlinestyles(h2,{':','--','-.'})
hold on
h_aux = semilogy(1:3,rand(3),'-o','linewidth',2);
setlinestyles(h_aux,{':','--','-.'})
set(h_aux,'visible','off')
h3 = semilogy(err_Gfwd_direct,'linewidth',2);
setlinestyles(h3,{':','--','-.'})
step = 6;
h4 = semilogy(5:step:n,err_Gfwd_direct(5:step:n,:),'o','markersize',7,'linewidth',2);
set(gca,'fontsize',16)
legend(alpha_strings_fwd,'location','best')
xlabel('k','fontsize',16)
ylabel('|a_k-b_k|','fontsize',16)
axis([0 100 1e-18 1e4])
saveas(22,['geronimus_fwd_3alpha',choice_string],'epsc')

end

%% 3. Forward conditioning (finish with choice = 1 in cell 2!)
% We generate two plots:
%
% 1) The error plot of the forward Geronimus algorithm in VPA. We compare
% with a reference sequence obtained with geronimus_bwd. We show errors for
% the following: 
%   * geronimus_bwd in DP, with other value of free parameter 
%   * geronimus_fwd_st in DP
%   * geronimus_fwd_st in VPA
%   * geronimus_fwd_st in VPA (except for one line)
%
% 2) Error plot of Christoffel transformation applied on
%   * geronimus_bwd DP output
%   * geronimus_fwd DP output
%   * geronimus_fwd VPA output
%   Compared with original sequence

fprintf('\n\n 3. Forward conditioning:')
fprintf('\n==========================\n\n')

alpha = alpha(3); % we picked this value of alpha

G1 = givensfactors([a;an(1)]);
G2 = givensfactors([a;an(2)]);

% Reference sequence by BWD on G1
Ghat = geronimus_bwd(G1,alpha);
ahat = reflection(Ghat);
% Free parameter for FWD algorithm
a1 = ahat(1);

G = givensfactors(a);
Gv = givensfactors(vpa(a));
% Geronimus Fwd in DP
Gfwd = geronimus_fwd_st(G,alpha,a1);
% Geronimus Fwd in VPA
fprintf('\n Geronimus Fwd in VPA ...')
tic
[Gfwd_vpa,k_breakdown] = geronimus_fwd_st(Gv,vpa(alpha),vpa(a1));
fprintf(' %0.3f seconds.\n',toc)
% Geronimus Fwd in VPA, except for one line (last argument = 1)
fprintf(' Geronimus Fwd in VPA (2) ...')
tic
Gfwd_semivpa = geronimus_fwd_st(Gv,vpa(alpha),vpa(a1),1);
fprintf(' %0.3f seconds.\n',toc)

% Extract Schur parameters
afwd = reflection(Gfwd);
afwd_vpa = reflection(Gfwd_vpa);
afwd_semivpa = reflection(Gfwd_semivpa);

% Errors on Schur parameters
err_fwd = abs(ahat-afwd);
err_fwd_vpa = abs(ahat-afwd_vpa);
err_fwd_semivpa = abs(ahat-afwd_semivpa);

% ---------------
% Bwd difference:
% ---------------

Ghat2 = geronimus_bwd(G2,alpha);
ahat2 = reflection(Ghat2);
diff_bwd = abs(ahat-ahat2);

% ---------------------------
% Christoffel on fwd outputs:
% ---------------------------

% Christoffel on output G.Fwd in DP
Gfwd_ch = christoffel(Gfwd,alpha);
% Christoffel on output G.Fwd in VPA
Gfwd_vpa_ch = christoffel(givensfactors(double(afwd_vpa)),alpha);
% Christoffel on output G.Fwd in VPA except for one line
Gfwd_semivpa_ch = christoffel(givensfactors(double(afwd_semivpa)),alpha);
% Christoffel on output G.Bwd
Gbwd_ch = christoffel(Ghat,alpha);

% Extract Schur parameters
afwd_ch = reflection(Gfwd_ch);
afwd_vpa_ch = reflection(Gfwd_vpa_ch);
afwd_semivpa_ch = reflection(Gfwd_semivpa_ch);
abwd_ch = reflection(Gbwd_ch);

% Errors on Schur parameters
err_fwd_ch = abs([a; an(1)] - afwd_ch);
err_fwd_vpa_ch = abs([a; an(1)] - afwd_vpa_ch);
err_fwd_semivpa_ch = abs([a; an(1)] - afwd_semivpa_ch);
err_bwd_ch = abs([a; an(1)] - abwd_ch);

% -----------
% --- plots: 
% -----------

figure(31),clf
oldCO = get(31,'DefaultAxesColorOrder');
set(31,'DefaultAxesColorOrder',[ 0 0 1 ; 0 0.5 0; 1 0 0; 0 0 0])
h1 = semilogy([err_fwd err_fwd_vpa err_fwd_semivpa diff_bwd],'linewidth',2);
setlinestyles(h1,{'-','--','-.',':'})
set(31,'DefaultAxesColorOrder',oldCO)
hold on
set(gca,'fontsize',16)
axis([0 100 1e-18 1e4])
xlabel('k','fontsize',16)
legend('errors fwd DP','errors fwd VPA','errors fwd partial VPA',...
    'differences bwd',...
    'location','east')
saveas(31,'geronimus_fwd_1alpha_vpa','epsc')

figure(33),clf
h2 = semilogy([err_fwd_ch err_fwd_vpa_ch err_bwd_ch],'linewidth',2);
setlinestyles(h2,{'-','--','-.'})
set(gca,'fontsize',16)
axis([0 100 1e-18 1e4])
xlabel('k','fontsize',16)
legend('Christoffel on fwd DP','Christoffel on fwd VPA',...
    'Christoffel on bwd',...
    'location','east')
saveas(33,'geronimus_fwd_1alpha_christoffel','epsc')


%% 4. Backward VPA and convex sets D (take choice = 1 in Cell 2!)
% Here we generate the following plots:
% * difference between two backward sequences in VPA, for one input
% sequence and one value of alpha. (Same as in Cell 3!)  
% * convex set of possible "positive definite Uvarov values" for m, given
% one of these backward sequences
% * set of possible values for the free parameter a1, given this same
% backward sequence
% * set of possible values for the free parameter a, but for truncations of
% the backward sequence. We truncate where the forward algorithm broke
% off in VPA in cell 3, see the variable k_breakdown.

fprintf('\n\n 4. Backward VPA and convex sets D:')
fprintf('\n==========================\n\n')

G1 = givensfactors([a;an(1)]);
G2 = givensfactors([a;an(2)]);
Gbwd1 = geronimus_bwd(G1,alpha);
Gbwd2 = geronimus_bwd(G2,alpha);
diff_Gbwd = abs(reflection(Gbwd1)-reflection(Gbwd2));

G1v = givensfactors(vpa([a;an(1)]));
G2v = givensfactors(vpa([a;an(2)]));
fprintf('Geronimus Bwd VPA...')
tic
Gbwd1v = geronimus_bwd(G1v,vpa(alpha));
fprintf('  %0.3f seconds.\n',toc)
fprintf('Geronimus Bwd VPA...')
tic
Gbwd2v = geronimus_bwd(G2v,vpa(alpha));
fprintf('  %0.3f seconds.\n',toc)
fprintf('Computing difference VPA...')
tic
diff_Gbwdv = abs(reflection(Gbwd1v)-reflection(Gbwd2v));
fprintf('  %0.3f seconds.\n',toc)

% -------------
% Region of m:
% -------------

r1 = double(reflection(Gbwd1));
c = r2c(1,r1);

m = valid_m_range(c,alpha,600);
% m Break Down Lower boundary
mBDL = valid_m_range(c(1:k_breakdown+1),alpha,600);
% m Break Down Higher boundary
mBDH = valid_m_range(c(1:k_breakdown+2),alpha,600);

u = alpha.^[0:1:n].';
v = alpha'.^[0:-1:-n].';
U = [u v];

T = toeplitz(c');

T2 = U'*T*U;
% calculate circle
t1 = real(T2(1,1));
t2 = T2(2,1);
t3 = real(T2(2,2));
tu = norm(u)^2;
tv = norm(v)^2;
dx = ( 2*real(t2)*(tu*tv+n^2)-2*n*(t1*tv+t3*tu) )/(n^2-tu*tv)^2;
dy = 2*imag(t2)/(n^2-tu*tv);
d0 = (abs(t2)^2-t1*t3)/(n^2-tu*tv)^2;

cR = sqrt(-d0+dx^2/4+dy^2/4);
cMx = -dx/2;
cMy = -dy/2;

% -------------
% Region of a1:
% -------------

a1 = r1(1);
diffa1 = (-a1*(m+conj(m))-m*alpha-conj(m)*alpha'^-1)./(1+m+conj(m));

% exact value of a1
a1e = -Gbwd1v{1}(1,1);
diffa1_BDL = (-a1*(mBDL+conj(mBDL))-mBDL*alpha-conj(mBDL)*alpha'^-1)./(1+mBDL+conj(mBDL));
diffa1_BDH = (-a1*(mBDH+conj(mBDH))-mBDH*alpha-conj(mBDH)*alpha'^-1)./(1+mBDH+conj(mBDH));

% -----------
% --- plots: 
% -----------

figure(41),clf
h1 = semilogy(diff_Gbwd,'b','linewidth',2);
hold on
h2 = semilogy(diff_Gbwdv,'r','linewidth',2);
setlinestyles([h1;h2],{'-','--'});
set(gca,'fontsize',16)
xlabel('k','fontsize',16)
ylabel('|a_k-b_k|','fontsize',16)
axis([0 100 1e-35 1e4])
legend('double precision','high precision','location','southeast')
saveas(41,'geronimus_bwd_1alpha_vpa','epsc')

figure(42),clf
% plot(m,'.r','markersize',12);
plot(m,'r','linewidth',2)
hold on
tt = linspace(0,2*pi,1000);
h_c = plot(cMx+cR*cos(tt),cMy+cR*sin(tt),'k--','linewidth',2);
axis equal
set(gca,'fontsize',16)
xlabel('Re(m)','fontsize',16)
ylabel('Im(m)','fontsize',16)
legend('convex set D','circle')
saveas(42,'uvarov_circles','epsc')

figure(43),clf
plot(diffa1,'r','linewidth',2)
xlabel('Re','fontsize',16)
ylabel('Im','fontsize',16)
axis equal
legend('a_1(m)-a_1^e')
set(gca,'fontsize',16)
saveas(43,'a1_circle','epsc')

figure(44),clf
plot(diffa1_BDL,'r','linewidth',2)
hold on
h3 = plot(diffa1_BDH,'linewidth',2);
set(h3,'color',[0 0.5 0])
setlinestyles(h3,{'--'})
plot(a1-a1e,'*b','markersize',15)
xlabel('Re','fontsize',16)
ylabel('Im','fontsize',16)
axis equal
legend(sprintf('a_1(m)-a_1^e for k = %i',k_breakdown),...
    sprintf('a_1(m)-a_1^e for k = %i',k_breakdown+1),...
    'a_1-a_1^e','location','southeast')
set(gca,'fontsize',16)
saveas(44,'a1_circle2','epsc')

%% 5. Compare Forward algorithms
% Here, we compare the errors between both forward algorithms. To make sure
% we get small Schur parameters in the resulting sequence, we use
% christoffel.m to compute the input sequence that will give a specific
% output sequence. 

fprintf('\n\n 5. Compare Forward algorithms:')
fprintf('\n==========================\n\n')


n = 200;
alpha = 1.001*exp(i*pi/5);
a = (0.1*rand(n,1)+0.01).*exp(i*2*pi*rand(n,1));
a(40) = 1e-4*exp(i*2*pi*rand);
a(80) = 1e-7*exp(i*2*pi*rand);
a(120) = 1e-10*exp(i*2*pi*rand);
a(160) = 1e-13*exp(i*2*pi*rand);

G = givensfactors(a);
Ghat = christoffel(G,alpha);

G1 = geronimus_fwd_direct(Ghat(1:n-1),alpha,a(1));
G2 = geronimus_fwd_st(Ghat(1:n-1),alpha,a(1));

a1 = reflection(G1);
a2 = reflection(G2);

Gbwd = geronimus_bwd(Ghat,alpha);
abwd = reflection(Gbwd);

% -----------
% --- plots: 
% -----------

figure(51),clf
h1 = semilogy([abs(a1-a) abs(a2-a) abs(abwd-a)],'linewidth',2);
setlinestyles(h1,{'none','-','--'},{'o','none','none'})
set(h1(1),'linewidth',1,'markersize',7)

xlabel('k','fontsize',16)
legend('error fwd direct','error fwd ST',...
    'error bwd',...
    'location','northwest')
set(gca,'fontsize',16)
saveas(51,'compare_fwd_errors','epsc')

figure(52)
semilogy(abs(a),'o','markersize',7)
set(gca,'fontsize',16)
xlabel('k','fontsize',16)
saveas(52,'compare_fwd_sequence','epsc')
