% author: Matthias Humet
% 
%
function [ posdef ] = moments_posdef( c)
%MOMENTS_POSDEF Given a vector of moments c, checks if the Toeplitz matrix
%   toeplitz(c') is positive definite.
%
% Uses Yule Walker scheme to compute reflection coefficients until a
% reflection coefficient has modulus larger than one. 

% if c(1) not positive real
if ~(abs(imag(c(1))) < 1e-14 && c(1) > 0)
    posdef = 0;
    return
end

n = length(c);
 
x = -c(2)/c(1);
r = x(1);
if abs(r) >= 1
    posdef = 0;
    return
end

for k = 1:n-2
    beta = ( -c(k+2) - c(2:k+1).'*x(1:k) )/( c(1) + x(k:-1:1)'*c(2:k+1) );
    x = beta*[1;conj(x(k:-1:1))] + [0;x]; 
    r = x(1);
    if abs(r) >= 1
        posdef = 0;
        return
    end
end

posdef = 1;

end

