% author: Matthias Humet
%
%
function [ c ] = r2c( c0,r )
%R2C Compute moments from Schur parameters by using the Szego recurrence
%relation and using only O(n) storage (useful for VPA). The
%algorithm itself still requires O(n^2) computations.

n = length(r);
c = [c0;r]; % c(2:end) will be overwritten
x = 1/sqrt(c0);
for k = 1:n
    rho = sqrt(1-abs(r(k))^2);
    x = 1/rho*([0;x]+r(k)*[flipud(conj(x));0]);
    c(k+1) = -1/x(k+1)*(x(1:k).'*c(1:k));
end

end

