% author: Matthias Humet
% 
%
function [ r ] = reflection( input )
%REFLECTION Compute reflection (Verblunsky) coefficients r. Input can be 
% the following :
%   * U:    triangular matrix with polynomials coefficients
%   * G:    cell array of 2x2 Givens transformations that form the unitary
%       Hessenberg matrix H associated with multiplication of z.

% input == U
if size(input,1) == size(input,2) && size(input,1) > 1
    % input nxn, n>1
    U = input;
    n = size(U,1);
    
    r = zeros(n-1,1);
    for k = 2:n
        r(k-1) = U(1,k)/U(k,k);
    end
% input == G
elseif size(input,1) == 1 || size(input,2) == 1
    % input length n, n >= 1
    G = input;
    n = length(G);
    
%     r = zeros(n,1);  comment this out to make VPA work (not much slower
%     anyway)
    for k = 1:n
        r(k,1) = -G{k}(1,1);
    end
else
    disp('** error reflection.m: Wrong input.')
end

end

